/*Copyright (C) 2010-2011, Mozilla Corporation.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
#if !defined(CELTPRIV_H)
# define CELTPRIV_H (1)
# include <math.h>
# include "celt.h"
# include "opuspriv.h"
# include "kiss_fft.h"
# include "celtdata.h"



typedef struct celt_mdct_config celt_mdct_config;



# define CELT_PI    (3.14159265F)
# define CELT_SQRT2 (1.41421356F)

# define CELT_EPSILON (1E-15)

# define CELT_BITRES        (3)
# define CELT_FINE_BITS_MAX (8)

# define CELT_SPREAD_NONE       (0)
# define CELT_SPREAD_LIGHT      (1)
# define CELT_SPREAD_NORMAL     (2)
# define CELT_SPREAD_AGGRESSIVE (3)

# define CELT_THETA_TRIANGLE_PDF (0)
# define CELT_THETA_UNIFORM_PDF  (1)
# define CELT_THETA_STEP_PDF     (2)

# define CELT_DECODE_BUFFER_SIZE (2048)
# define CELT_MAX_PERIOD         (1024)
# define CELT_LPC_ORDER          (24)

# define CELT_COSF(_x)      ((float)cos(_x))
# define CELT_SINF(_x)      ((float)sin(_x))
# define CELT_SQRTF(_x)     ((float)sqrt(_x))
# define CELT_RSQRTF(_x)    (1.0F/CELT_SQRTF(_x))
# define CELT_EXP2F(_x)     ((float)exp(0.69314718F*(_x)))



struct celt_mdct_config{
  /*The number of (frequency-domain) MDCT bins in the smallest frame size.*/
  int                   short_size;
  /*The amount of overlap (in samples) at all frame sizes.*/
  int                   overlap;
  /*The shift needed to index the MDCT twiddle array at the smallest frame
     size.
    This allows the same twiddles to be shared across multiple configurations.*/
  int                   tw_shift;
  /*The first half of the symmetric MDCT window.*/
  const float          *window;
  /*twiddles[k<<tw_shift]=cos(2*pi*(k/N)), where N==2*short_size, and k ranges
     from 0 to N/4 (inclusive).*/
  const float          *twiddles;
  /*The FFT configuration for each frame size scale (lm in 0...lm_max).*/
  const kiss_fft_cfg    kfft[4];
};


/*The number of pulses to allocate to a PVQ codeword with index _k.
  CELT limits the number of pulses in a PVQ codeword to a small subset of the
   possible values (even though we could decode one with any arbitrary amount),
   in order to cache the size (in bits) of each codeword in a small lookup
   table, saving considerable computation during bit allocation.
  The values in this subset are spaced so that they're exponentially
   increasing, which makes them approximately uniformly distributed with
   respect to the number of bits they use.*/
#define CELT_PULSE_COUNT(_k) ((_k)<8?(_k):8+((_k)&7)<<((_k)>>3)-1)


struct celt_mode{
  /*The sampling rate, in Hz.*/
  opus_int32           sample_rate;
  /*The number of bands in the bitstream.*/
  int                  nebands;
  /*The maximum frame size scaling (inclusive).*/
  int                  lm_max;
  /*The number of rows in the bit allocation matrix.*/
  int                  nalloc_lines;
  /*The pre-emphasis filter taps.*/
  float                preemph[4];
  /*The boundaries of each band at the smallest frame size: band bi contains
     MDCT bins ebands[bi]...(ebands[bi+1]-1).*/
  const unsigned char *ebands;
  /*The reciprocal of the square root of the size of each band at the smallest
     frame size, i.e., 1/sqrt(ebands[bi+1]-ebands[bi]).*/
  const float         *rsqrt_n0;
  /*The allocation matrix: each row has nebands values, which are the bits per
     sample in units of 1/32nd bits.*/
  const unsigned char *alloc_lines;
  /*The allocation caps, i.e., the maximum number of bits such that, when we
     allocate that many to a band, it will reliably use them all.
    Each row has nebands values, with two rows for each split level: one for
     mono and one for stereo, which are bits per sample in units of
     1/32nd bits, minus 64.*/
  const unsigned char *alloc_caps;
  /*The index of the cache line in bits[] for each band and split level.
    The split level, lm, can range from -1 to lm_max (inclusive), and the index
     of the corresponding line for band bi is found at position
     index[(lm+1)*nebands+bi].*/
  const opus_int16    *cache_index;
  /*The cache lines for PVQ bit usage.
    The first entry in each cache line is the total number of following entries.
    Each following entry, k, is the bit usage, in 1/8th bits, minus 1/8th bit,
     of a PVQ codeword with CELT_PULSE_COUNT(k) pulses, with k starting at 1
     and going up to a maximum of 40 (128 pulses).*/
  const unsigned char *cache_bit8s;
  celt_mdct_config     mdct;
};



struct celt_decoder{
  const celt_mode *mode;
  int              stereo;
  int              postfilter_period1;
  int              postfilter_period2;
  int              postfilter_tapset1;
  int              postfilter_tapset2;
  float            postfilter_gain1;
  float            postfilter_gain2;
  float            preemph_mem[2];
  float           *decode_mem[2];
  float           *band_log_e;
  float           *hist_log_e0;
  float           *hist_log_e1;
  opus_uint32      rng;
};


int celt_decode_impl(celt_decoder *_dec,opus_range_coder *_rc,
 float *_pcm,int _nchannels,int _b0,int _b_end,int _lm);

#endif
